/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.groovy

import org.codehaus.groovy.runtime.InvokerHelper

import spock.lang.Specification

class GroovyMopExploration extends Specification {
  def "call method that takes a String with GString through InvokerHelper"() {
    def target = new GroovyMopExploration()
    def arg = "one ${"t" + "w" + "o"} three"

    expect:
    InvokerHelper.invokeMethod(target, "foo", [arg] as Object[]) == "one two three"
  }

  def "call method that takes a String with GString through MetaMethod"() {
    def target = new GroovyMopExploration()
    def arg = "one ${"t" + "w" + "o"} three"

    expect:
    def metaClass = InvokerHelper.getMetaClass(target)
    def metaMethod = metaClass.pickMethod("foo", [arg.getClass()] as Class[])
    metaMethod.invoke(target, [arg] as Object[]) == "one two three"
  }

  def foo(String str) { str }
}