/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.util.inspector;

/**
 * Unchecked exception thrown by AstInspector when:
 * <ul>
 * <li>an IOException occurs when loading a file</li>
 * <li>a declaration with the specified name cannot be found</li>
 * <li>an internal error occurs</li>
 * </ul>
 *
 * @author Peter Niederwieser (pniederw@gmail.com)
 */
public class AstInspectorException extends RuntimeException {
  public AstInspectorException(String msg) {
    super(msg);
  }

  public AstInspectorException(String msg, Exception cause) {
    super(msg, cause);
  }
}
