/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.util

class LinkedHashMultisetSpec extends AbstractMultisetSpec {
  def <T> IMultiset<T> createMultiset(List<T> elements) {
    new LinkedHashMultiset<T>(elements)
  }

  def "iteration order follows order of first insertion of an element"() {
    multiset = createMultiset(["a", "b", "a", "c" , "d", "c", "e"])

    expect:
    toList(multiset.iterator()) == ["a", "b", "c", "d", "e"]
    multiset.toArray() == ["a", "b", "c", "d", "e"] as Object[]
    toList(multiset.entrySet().collect { it.key }.iterator()) == ["a", "b", "c", "d", "e"]
  }

  private toList(Iterator iterator) {
    def result = []
    while (iterator.hasNext()) {
      result << iterator.next()
    }
    result
  }
}
