/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */  

package org.spockframework.spring

import org.springframework.beans.factory.annotation.Autowired
import org.springframework.context.ApplicationContext
import org.springframework.test.annotation.DirtiesContext
import org.springframework.test.context.ContextConfiguration

import spock.lang.*

@ContextConfiguration(locations = "InjectionExamples-context.xml")
class DirtiesContextExample extends Specification {
  @Shared savedContext

  @Autowired
  ApplicationContext context

  @DirtiesContext
  def "first spec method"() {
    setup:
    savedContext = context
  }

  def "second spec method"() {
    expect:
    !context.is(savedContext)
  }
}
