/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.smoke.condition

import spock.lang.Specification
import org.spockframework.runtime.UnallowedExceptionThrownError
import spock.lang.FailsWith

class NegativeExceptionConditions extends Specification {
  def "no exception allowed, none thrown"() {
    when:
    def x = 1

    then:
    noExceptionThrown()
  }

  @FailsWith(UnallowedExceptionThrownError)
  def "no exception allowed, IOException thrown"() {
    when:
    throw new IOException()

    then:
    noExceptionThrown()
  }

  def "no IOException allowed, none thrown"() {
    when:
    def x = 1

    then:
    notThrown(IOException)
  }

  @FailsWith(UnallowedExceptionThrownError)
  def "no IOException allowed, IOException thrown"() {
    when:
    throw new IOException()

    then:
    notThrown(IOException)
  }

  @FailsWith(RuntimeException)
  def "no IOException allowed, other exception thrown"() {
    when:
    throw new RuntimeException()

    then:
    notThrown(IOException)
  }
}
