/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */  

package org.spockframework.spring

import org.springframework.beans.factory.annotation.Autowired
import org.springframework.test.context.ContextConfiguration

import spock.lang.*

abstract class BaseSpec extends Specification {
  @Autowired
  IService1 service1

  def setup() {
    assert service1 instanceof IService1
    assert this.service2 instanceof IService2
  }
}

@ContextConfiguration(locations = "InjectionExamples-context.xml")
class SpringSpecInheritance extends BaseSpec {
  @Autowired
  IService2 service2

  def setup() {
    assert service1 instanceof IService1
    assert service2 instanceof IService2
  }

  def "fields of base class have been injected"() {
    expect:
    service1 instanceof IService1

  }

  def "fields of derived class have been injected"() {
    expect:
    service2 instanceof IService2
  }
}
