/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.smoke

import org.spockframework.EmbeddedSpecification
import org.spockframework.compiler.InvalidSpecCompileException

/**
 * @author Peter Niederwieser
 */

class ExpectBlocks extends EmbeddedSpecification {
  def "may not contain exception conditions"() {
    when:
    compiler.compileFeatureBody("""
expect: thrown(RuntimeException)
    """)

    then:
    thrown(InvalidSpecCompileException)
  }

  def "may not contain interactions"() {
    when:
    compiler.compileFeatureBody("""
def l = Mock(List)
expect: l.size() >> 5
    """)

    then:
    InvalidSpecCompileException e = thrown()
    e.line == 2
  }
}