/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.util

import spock.lang.Specification

class TextUtilSpec extends Specification {
  def "camelCaseToConstantCase"() {
    expect:
    TextUtil.camelCaseToConstantCase(camelCase) == constantCase

    where:
    camelCase      | constantCase
    ""             | ""
    "do"           | "DO"
    "DO"           | "DO"
    "doIt"         | "DO_IT"
    "DoIt"         | "DO_IT"
    "doIT"         | "DO_IT"
    "DOit"         | "D_OIT"
    "useTCPStack"  | "USE_TCP_STACK"
    "useTCPStacK"  | "USE_TCP_STAC_K"
    "fred123Usage" | "FRED123_USAGE"
  }
}
