/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.runtime.model;

/**
 * The different kinds of methods that a <tt>MethodInfo</tt> instance can represent.
 *
 * @author Peter Niederwieser
 */
public enum MethodKind {
  INITIALIZER,
  SHARED_INITIALIZER,
  SETUP,
  CLEANUP,
  SETUP_SPEC,
  CLEANUP_SPEC,
  FEATURE,
  DATA_PROVIDER,
  DATA_PROCESSOR,
  SPEC_EXECUTION,
  FEATURE_EXECUTION,
  ITERATION_EXECUTION;

  public boolean isFixtureMethod() {
    return isSetupMethod() || isCleanupMethod();
  }
  
  public boolean isSetupMethod() {
    return this == SETUP || this == SETUP_SPEC;
  }

  public boolean isCleanupMethod() {
    return this == CLEANUP || this == CLEANUP_SPEC;
  }

  public boolean isFeatureScopedFixtureMethod() {
    return this == SETUP || this == CLEANUP;
  }

  public boolean isSpecScopedFixtureMethod() {
    return this == SETUP_SPEC || this == CLEANUP_SPEC;
  }
}
