/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.runtime;

/**
 * Indicates that an operation timed out.
 */
public class SpockTimeoutError extends SpockAssertionError {
  private final double timeout;

  public SpockTimeoutError(double timeout, String message) {
    this(timeout, message, null);
  }

  public SpockTimeoutError(double timeout, String message, Throwable cause) {
    super(message, cause);
    this.timeout = timeout;
  }

  /**
   * Returns the timeout (in seconds).
   *
   * @return the timeout (in seconds)
   */
  public double getTimeout() {
    return timeout;
  }
}
