/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.runtime;

/**
 *
 * @author Peter Niederwieser
 */
public class SpockAssertionError extends AssertionError {
  public SpockAssertionError() {}

  public SpockAssertionError(String message) {
    super(message);
  }

  public SpockAssertionError(Throwable cause) {
    initCause(cause);
  }

  public SpockAssertionError(String message, Throwable cause) {
    super(message);
    initCause(cause);
  }

  // We don't want to have the class name printed out for assertion errors, so
  // we just delegate to getMessage().
  @Override
  public String toString() {
    return getMessage();
  }
}
