/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.mock.response;

import org.spockframework.mock.IChainableResponseGenerator;
import org.spockframework.mock.IMockInvocation;

public abstract class SingleResponseGenerator implements IChainableResponseGenerator {
  private boolean endOfCycle = false;

  public boolean isAtEndOfCycle() {
    return endOfCycle;
  }

  public final Object respond(IMockInvocation invocation) {
    endOfCycle = true;
    return doRespond(invocation);
  }
  
  public abstract Object doRespond(IMockInvocation invocation);
}
