/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.smoke

import spock.lang.Shared
import spock.lang.Specification

/**
 * A ...
 
 * @author Peter Niederwieser
 */
class MethodAccessibility extends Specification {
  @Shared numCalls = 0

  private setup() {
    numCalls++
  }

  protected cleanup() {
    numCalls++
  }

  private setupSpec() {
    numCalls++
  }

  def cleanupSpec() {
    assert numCalls == 4
  }

  private "a feature"() {
    numCalls++
    expect: true
  }
}