/*
 * Copyright 2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.spockframework.groovy

import org.spockframework.util.inspector.AstInspector
import spock.lang.Specification

/**
 * @author Peter Niederwieser
 */
class PackageNames extends Specification {
  def inspector = new AstInspector()

  def "package names and imports end in '.'"() {
inspector.load("""
package foo.bar

import java.lang.Class
import java.util.*
""")

    def m = inspector.module

    expect:
    m.packageName == "foo.bar."
    m.starImports[0].packageName == "java.util."
  }
}